package CDMA;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.Math;
import Library.*;
import Library.Control.*;

public class CDMAControlBar extends ControlBar {
	CDMA             parent;
	CDMALayer        layer;
	CDMAGuide        guide;
	CDMACode         code;
	CDMAParameters   P;
	LibraryStrings   LS;
	CDMAGuideStrings GS;
	CDMAStrings      S;

	StandardChoice   guideChoice;
	StandardChoice   guideSectionChoice;
	StandardButton   guideStepButton;
        StandardButton   guideBackButton;

	StandardCheckbox showPlane;
	StandardCheckbox showPoints;
	StandardCheckbox showAxes;
	StandardCheckbox showVectors;
	StandardCheckbox showVector;
	StandardCheckbox showNormal;
	StandardCheckbox showSignatures;
	StandardCheckbox showAmplitudes;
	StandardCheckbox periodic;
	StandardCheckbox sgnOutline;
	StandardCheckbox user0;
	StandardCheckbox user1;
	StandardCheckbox user2;
	StandardLabel    NLabel;
	StandardIntField NField;
	StandardLabel    LLabel;
	StandardIntField LField;
	StandardLabel    PLabel;
	StandardIntField PField;
	StandardLabel    noiseLabel;
	StandardIntField noiseField;
        StandardButton   zeroNoise;
        StandardButton   lowNoise;
        StandardButton   highNoise;

public void setup(UniString name, Rectangle rectangle) {
	super.setup(name,rectangle);
	LS        = parent.LS;

	add(guideChoice        = new StandardChoice(this));
		guideChoice.addItem(LS.guide);
		guideChoice.addItem(LS.work);
	add(guideSectionChoice = new StandardChoice(this));
		guideSectionChoice.addItem(GS.signatures);
		guideSectionChoice.addItem(GS.signal);
		guideSectionChoice.addItem(GS.signatures2D);
		guideSectionChoice.addItem(GS.signal2D);
		guideSectionChoice.addItem(GS.samples2D);
		guideSectionChoice.addItem(GS.decoding2D);
		guideSectionChoice.addItem(GS.signatures3D);
		guideSectionChoice.addItem(GS.signal3D);
		guideSectionChoice.addItem(GS.samples3D);
		guideSectionChoice.addItem(GS.decoding3D);
	add(guideStepButton     = new StandardButton(this,LS.nextScene));
        add(guideBackButton     = new StandardButton(this,LS.previousScene));

	add(showPlane      = new StandardCheckbox(this,S.showPlane,     true));
	add(showPoints     = new StandardCheckbox(this,S.showPoints,    true));
	add(showAxes       = new StandardCheckbox(this,S.showAxes,      true));
	add(showVectors    = new StandardCheckbox(this,S.showVectors,   true));
	add(showVector     = new StandardCheckbox(this,S.showVector,    true));
	add(showNormal     = new StandardCheckbox(this,S.showNormal,    true));
	add(showSignatures = new StandardCheckbox(this,S.showSignatures,true));
	add(showAmplitudes = new StandardCheckbox(this,S.showAmplitudes,true));
	add(periodic       = new StandardCheckbox(this,S.periodic,      false));
	add(sgnOutline     = new StandardCheckbox(this,S.sgnOutline,    true));
	add(user0          = new StandardCheckbox(this,S.user0,         true));
	add(user1          = new StandardCheckbox(this,S.user1,         false));
	add(user2          = new StandardCheckbox(this,S.user2,         false));

	add(NLabel         = new StandardLabel(this,S.N));
	add(NField         = new StandardIntField(this,3,3));
	add(LLabel         = new StandardLabel(this,S.L));
	add(LField         = new StandardIntField(this,3,3));
	add(PLabel         = new StandardLabel(this,S.P));
	add(PField         = new StandardIntField(this,32,3));
	add(noiseLabel     = new StandardLabel(this,S.noiseMagnitude));
	add(noiseField     = new StandardIntField(this,25,3));
        add(zeroNoise      = new StandardButton(this,S.zero));
        add(lowNoise       = new StandardButton(this,S.low));
        add(highNoise      = new StandardButton(this,S.high));
	}

public void placeFull() {
	placeElement(guideChoice);
	if (guideChoice.getSelectedIndex() == 0) {
		placeElement(guideBackButton,guideStepButton);
		placeElement(guideSectionChoice);
		placeSpace();
		guide.place(true);
		return;
		};

	placeElement(showPlane);
	placeElement(showPoints);
	placeElement(showAxes);
	placeElement(showVectors);
	placeElement(showVector);
	placeElement(showNormal);
	placeElement(showSignatures);
	placeElement(showAmplitudes);
	placeElement(periodic);
	placeElement(sgnOutline);
	placeElement(user0);
	placeElement(user1);
	placeElement(user2);
	placeElement(NLabel,NField);
	placeElement(LLabel,LField);
	placeElement(PLabel,PField);
	placeElement(noiseLabel,noiseField);
        placeElement(zeroNoise,lowNoise,highNoise);
	}

public void placeSmall() {placeFull();}

public void actionPerformed(ControlElement e) {
	super.actionPerformed(e);
	if(e == guideSectionChoice) {
		if(guideSectionChoice.getSelectedItem()      == GS.signatures)
			guide.nextScene(guide.signatures);
		else if(guideSectionChoice.getSelectedItem() == GS.signal)
			guide.nextScene(guide.signal);
		else if(guideSectionChoice.getSelectedItem() == GS.signatures2D)
			guide.nextScene(guide.signatures2D);
		else if(guideSectionChoice.getSelectedItem() == GS.signal2D)
			guide.nextScene(guide.signal2D);
		else if(guideSectionChoice.getSelectedItem() == GS.samples2D)
			guide.nextScene(guide.samples2D);
		else if(guideSectionChoice.getSelectedItem() == GS.decoding2D)
			guide.nextScene(guide.decoding2D);
		else if(guideSectionChoice.getSelectedItem() == GS.signatures3D)
			guide.nextScene(guide.signatures3D);
		else if(guideSectionChoice.getSelectedItem() == GS.signal3D)
			guide.nextScene(guide.signal3D);
		else if(guideSectionChoice.getSelectedItem() == GS.samples3D)
			guide.nextScene(guide.samples3D);
		else if(guideSectionChoice.getSelectedItem() == GS.decoding3D)
			guide.nextScene(guide.decoding3D);
		}
        else if(e == guideStepButton) {
		guide.nextScene(true);
		}
        else if(e == guideBackButton) {
		guide.nextScene(false);
		}
	else if(e==user0) {
		layer.signals.activeUser = 0;
		user0.setState(true);
		user1.setState(false);
		user2.setState(false);
		}
	else if(e==user1) {
		layer.signals.activeUser = 1;
		user0.setState(false);
		user1.setState(true);
		user2.setState(false);
		}
	else if(e==user2) {
		layer.signals.activeUser = 2;
		user0.setState(false);
		user1.setState(false);
		user2.setState(true);
		}
	else if(e==NField) {
		int N =  NField.getIntValue();
		int L =  LField.getIntValue();
		if(N<2) N = 2;
		if(N>8) N = 8;
		if(L<N) L = N;
		NField.setIntValue(N);
		LField.setIntValue(L);
		layer.reset();
		}
	else if(e==LField) {
		int N =  NField.getIntValue();
		int L =  LField.getIntValue();
		if(L< N) L =  N;
		if(L>16) L = 16;
		LField.setIntValue(L);
		layer.reset();
		}
	else if(e==PField) {
		int L = LField.getIntValue();
		int P = PField.getIntValue();
		if(P<1)     P = 1;
		if(L*P>256) P = 256/L;
		PField.setIntValue(P);
		layer.reset();
		}
	else if(e==zeroNoise) {
		layer.setNoise(0);
		}
	else if(e==lowNoise) {
		layer.setNoise(1);
		}
	else if(e==highNoise) {
		layer.setNoise(2);
		};
	}

public void setDisabled() {
	guideBackButton.setDisabled(guide.scene <= 1);
	guideStepButton.setDisabled(guide.scene >= guide.lastScene);
	}

}

