package CDMA;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.Math;
import Library.*;
import Library.Geometry.*;

public class CDMALayer extends Layer {
	CDMA           parent;
	CDMAControlBar control;
	CDMAGuide      guide;
	CDMACode       code;
	CDMAParameters P;
	LibraryStrings LS;
	CDMAStrings    S;

	CDMA3D         signals;
	int            currentSample;
	CDMASgnData    plot, geom;
	double         sgnRight;
	double         geomLeft;

	int            N;  // number of users
	int            L;  // width of signature
	int            M;  // number of samples
	int            Q;  // number of visible samples


public void setup(UniString name, Rectangle rectangle) {
	super.setup(name,rectangle);
	reset();	
	}

double noiseMagnitude() {double result = control.noiseField.getIntValue(); result /= 100; return result;}

void reset() {
	N = control.NField.getIntValue();
	L = control.LField.getIntValue();
	M = 1000;
	Q = control.PField.getIntValue();
	sgnRight = 0.25;
	geomLeft = 0.30;
	initPlotData();
	initSgnData();
	if(guide.plot()) signals = new CDMA3D(this);
	else             signals = new CDMA3D(this);
	currentSample = -1;
	}

CDMASgnData sgnData() {
	if(guide.plot()) return plot; else return geom;
	}

void initPlotData() {
	R2D rect = pb.getRectangle();
	plot        = new CDMASgnData();

	plot.x        = rect.x + 0.05 + 0.25 / N;
	double right  = rect.x + rect.w; 
	plot.w        = (right - plot.x ) / (L*Q);

	double bordy  = 0.01;
	double top    = rect.y + 0.075;
	double bottom = rect.y + rect.h - bordy;
	plot.sigh     = (bottom - top) / 8;
	double middle = bottom - 2*plot.sigh;
	plot.sigy     = bottom - plot.sigh;
	plot.spacey   = (middle - top) / (2*N);
	plot.sgnh     = (plot.spacey - bordy) / 2;
	plot.sgny     = top + plot.sgnh;
	}

void initSgnData() {
	R2D rect = pb.getRectangle();
	geom        = new CDMASgnData();

	geom.x        = rect.x + 0.05 + 0.25 / N;
	double right  = sgnRight; 
	geom.w        = (right - geom.x ) / (L*Q);

	double bordy  = 0.01;
	double top    = rect.y + 0.075;
	double bottom = rect.y + rect.h - bordy;
	geom.sigh     = (bottom - top) / 8;
	double middle = bottom - 2*geom.sigh;
	geom.sigy     = bottom - geom.sigh;
	geom.spacey   = (middle - top) / (2*N);
	geom.sgnh     = (geom.spacey - bordy) / 2;
	geom.sgny     = top + geom.sgnh;
	}

boolean isInSignatureField(C2D mousePosition) {
	if(control.showSignatures.getState())
		if(mousePosition.x < geomLeft) return true;
	return false;
	}

void checkSample2D(C2D mousePosition) {
	currentSample = -1;
	for(int m=1; m<M; m++) {
		C2D s = signals.transform2D(signals.receivedC2D(m));
		double dx = mousePosition.x - s.x;
		double dy = mousePosition.y - s.y;
		if(dx*dx+dy*dy < P.sampleR*P.sampleR) currentSample = m;
		};
	if(currentSample>0) {
		signals.amplitude  [0][0].value = signals.amplitude  [0][currentSample].value;
		signals.amplitude  [1][0].value = signals.amplitude  [1][currentSample].value;
		signals.transmitted[0][0].value = signals.transmitted[0][currentSample].value;
		signals.transmitted[1][0].value = signals.transmitted[1][currentSample].value;
		signals.received   [0][0].value = signals.received   [0][currentSample].value;
		signals.received   [1][0].value = signals.received   [1][currentSample].value;
		};
	}

void checkSample3D(C2D mousePosition) {
	currentSample = -1;
	for(int m=1; m<M; m++) {
		C2D s = signals.transformToScreen(signals.receivedV3D(m));
		double dx = mousePosition.x - s.x;
		double dy = mousePosition.y - s.y;
		if(dx*dx+dy*dy < P.sampleR*P.sampleR) currentSample = m;
		};
	if(currentSample>0) {
		signals.amplitude  [0][0].value = signals.amplitude  [0][currentSample].value;
		signals.amplitude  [1][0].value = signals.amplitude  [1][currentSample].value;
		signals.amplitude  [2][0].value = signals.amplitude  [2][currentSample].value;
		signals.transmitted[0][0].value = signals.transmitted[0][currentSample].value;
		signals.transmitted[1][0].value = signals.transmitted[1][currentSample].value;
		signals.transmitted[2][0].value = signals.transmitted[2][currentSample].value;
		signals.received   [0][0].value = signals.received   [0][currentSample].value;
		signals.received   [1][0].value = signals.received   [1][currentSample].value;
		signals.received   [2][0].value = signals.received   [2][currentSample].value;
		};
	}

void setNoise(int level) {
	int noise = 0;
	if(guide.geo2D()) {
		if(level == 1) noise = 20;
		if(level == 2) noise = 40;
		}
	else	{
		if(level == 1) noise = 20;
		if(level == 2) noise = 40;
		};
	control.noiseField.setIntValue(noise);
	}

void setCursor(boolean resizeCursor) {
	if(resizeCursor) { //zmen kurzor na resize
		screen.setCursor(new Cursor(Cursor.N_RESIZE_CURSOR));
		}
	else	{ //zmen kurzor na default
		screen.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		};
	}

public boolean mouseAction0(MouseEvent evt, C2D mousePosition, boolean setValue) {
	boolean result = false;
	if(isInSignatureField(mousePosition)) {
		for(int n=0; n<N; n++) {
			for(int l=0; l<L; l++)
				if(signals.signature[n][l][0].mouseAction(mousePosition,setValue))
					result = true;
			if(signals.amplitude[n][0].mouseAction(mousePosition,setValue))
					result = true;
			};
		};
	setCursor(result);
	return result;
	}

public boolean mouseAction(MouseEvent evt, C2D mousePosition, boolean setValue) {
	boolean result = false;
	if(!parent.control.periodic.getState()) {
		if(isInSignatureField(mousePosition)) Q = 1;
		else return false;
		};
	for(int n=0; n<N; n++)
	for(int l=0; l<L; l++)
	for(int m=0; m<M; m++)
		if(signals.signature[n][l][m].mouseAction(mousePosition,setValue)) {
			result = true;
			if(setValue) for(int q=0; q<M; q++)
				signals.signature[n][l][q].value =
					signals.signature[n][l][m].value;
			};
	for(int n=0; n<N; n++)
	for(int m=0; m<M; m++)
		if(signals.amplitude[n][m].mouseAction(mousePosition,setValue)) result = true;
	setCursor(result);
	return result;
	}

public boolean mouseMoved(MouseEvent evt, C2D mousePosition) {
	return mouseAction(evt,mousePosition,false);
	}

public boolean mousePressed(MouseEvent evt, C2D mousePosition) {
	if(parent.control.periodic.getState()) return false;
	if(isInSignatureField(mousePosition))  return false;
	if(control.showPoints.getState()) if(rightButton(evt)) {
		if(guide.geo2D()) checkSample2D(mousePosition);
		if(guide.geo3D()) checkSample3D(mousePosition);
		};
	int button = evt.getButton();
	if(guide.geo2D()) if(leftButton(evt)) {
		signals.separate(mousePosition);
		return true;
		}
	if(rightButton(evt)) return signals.planeObserver.mousePressed(mousePosition);
	return signals.observer.mousePressed(mousePosition);
	}

public boolean mouseDragged(MouseEvent evt, C2D mousePosition) {
	if(guide.plot()||isInSignatureField(mousePosition))
		return mouseAction(evt,mousePosition,true);
	if(guide.geo2D()) if(leftButton(evt)) {
		signals.separate(mousePosition);
		return true;
		}
	if(guide.geo3D()) {
		if(leftButton(evt)) return signals.observer.mouseDragged(mousePosition);
		return signals.planeObserver.mouseDragged(mousePosition);
		}
	return false;
	}

public boolean mouseReleased(MouseEvent evt, C2D mousePosition) {
	setCursor(false);
	return true;
	}

public void paint() {
	signals.paint(pb);
	}

}

