package Delis;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.Math;
import Library.*;
import Library.Control.*;
import Library.Geometry.*;

public class DelisControlBar extends ControlBar {
	Delis              parent;
	DelisLayer         layer;
	DelisGuide         guide;
	DelisParameters    P;
	DelisStrings       S;
	DelisGuideStrings  GS;
	LibraryStrings     LS;

	StandardChoice   guideChoice;
	StandardChoice   guideSectionChoice;
	StandardButton   guideStepButton;
        StandardButton   guideBackButton;


	StandardButton   morePowerButton;
	StandardButton   lessPowerButton;

	StandardChoice   modelChoice;

	StandardChoice   inputChoice;
	StandardButton   newInput;

	StandardLabel    nLabel;

	StandardIntField nField;
	StandardLabel    biasedLabel;
	StandardIntField biasedField;
	StandardLabel    clusterLabel;
	StandardIntField clusterField;

	StandardCheckbox showTransmit;
	StandardCheckbox showEdges;



	StandardCheckbox showLattice;
	StandardLabel    latticeBoxLabel;

	StandardIntField latticeBoxField;
	StandardChoice   latticeBoxSelectChoice;
	StandardChoice   latticeBoxDisplayChoice;

	StandardCheckbox computeBridges;
	StandardButton   bridgeButton;
	StandardButton   phaseButton;

	StandardCheckbox showPowerPlot;
	StandardCheckbox showCongestionPlot;

	StandardCheckbox showExplanation;

public void setup(UniString name, Rectangle rectangle) {
	super.setup(name,rectangle);
	LS = screen.LS;
	R2D r = pb.getRectangle();
	barPosition = new C2D(r.x+r.w-0.3,0.15);

	add(guideChoice        = new StandardChoice(this));
		guideChoice.addItem(LS.guide);
		guideChoice.addItem(LS.work);
	add(guideSectionChoice = new StandardChoice(this));
		guideSectionChoice.addItem(GS.scene[guide.opening]);
		guideSectionChoice.addItem(GS.scene[guide.randomRadius]);
		guideSectionChoice.addItem(GS.scene[guide.uniformRadius]);
		guideSectionChoice.addItem(GS.scene[guide.uniformRadiusBiased]);
		guideSectionChoice.addItem(GS.scene[guide.uniformRadiusLattice]);
		guideSectionChoice.addItem(GS.scene[guide.NNG]);
		guideSectionChoice.addItem(GS.scene[guide.NNGBiased]);
		guideSectionChoice.addItem(GS.scene[guide.NNGStrongBiased]);
		guideSectionChoice.addItem(GS.scene[guide.lowDegree]);
		guideSectionChoice.addItem(GS.scene[guide.lowDegreeStrongBiased]);
		guideSectionChoice.addItem(GS.scene[guide.bridge]);
		guideSectionChoice.addItem(GS.scene[guide.bridgePlot]);
		guideSectionChoice.addItem(GS.scene[guide.bridgeScream]);

	add(guideStepButton     = new StandardButton(this,LS.nextScene));
        add(guideBackButton     = new StandardButton(this,LS.previousScene));

	add(morePowerButton          = new StandardButton(this,S.morePower));
        add(lessPowerButton         = new StandardButton(this,S.lessPower));

	add(modelChoice 	= new StandardChoice(this));
		modelChoice.addItem(S.randomRadius);
		modelChoice.addItem(S.uniformRadius);
		modelChoice.addItem(S.nng);
		modelChoice.addItem(S.lowDegree);

	add(inputChoice 	= new StandardChoice(this));
		inputChoice.addItem(S.uniformlyRandom);
		inputChoice.addItem(S.isolatedPoint);
		inputChoice.addItem(S.isolatedCluster);
	add(newInput            = new StandardButton(this,S.newInput));
	add(nLabel              = new StandardLabel(this,S.n));
	add(nField              = new StandardIntField(this,100,4));
	add(biasedLabel         = new StandardLabel(this,S.biasedLabel));
	add(biasedField         = new StandardIntField(this,30,2));
	add(clusterLabel        = new StandardLabel(this,S.clusterLabel));
	add(clusterField        = new StandardIntField(this,10,2));

	add(showTransmit        = new StandardCheckbox(this,S.showTransmit,true));
	add(showEdges           = new StandardCheckbox(this,S.showEdges,true));


	add(showLattice             = new StandardCheckbox(this,S.showLattice,false));
	add(latticeBoxLabel         = new StandardLabel(this,S.nBoxes));

	add(latticeBoxField         = new StandardIntField(this,16,2));
	add(latticeBoxSelectChoice  = new StandardChoice(this));
		latticeBoxSelectChoice.addItem(S.nonemptyCenter);
		latticeBoxSelectChoice.addItem(S.clusterCenter);
	add(latticeBoxDisplayChoice = new StandardChoice(this));
		latticeBoxDisplayChoice.addItem(S.displayAll);
		latticeBoxDisplayChoice.addItem(S.displaySelected);

       add(computeBridges      = new StandardCheckbox(this,S.computeBridges,false));
       add(bridgeButton        = new StandardButton(this,S.bridge));
       add(phaseButton         = new StandardButton(this,S.phase));

	add(showPowerPlot       = new StandardCheckbox(this,S.showPowerPlot,false));
	add(showCongestionPlot  = new StandardCheckbox(this,S.showCongestionPlot,false));
	add(showExplanation     = new StandardCheckbox(this,S.showExplanation,true));
	}

public void place() {
	if(guideSectionChoice.getSelectedItem() == GS.scene[guide.opening]) {
		place0();
		placeElement(guideStepButton);
		}
	else {super.place();};
	}

public void placeFull() {
	placeElement(guideChoice);
	if (guideChoice.getSelectedIndex() == 0) {
		placeElement(guideBackButton,guideStepButton);
		placeElement(guideSectionChoice);
		placeSpace();
		guide.place(true);
		return;
		};

	placeElement(lessPowerButton,morePowerButton);

	placeElement(modelChoice);

	placeElement(inputChoice);
	placeElement(newInput);
	placeElement(nLabel,nField);
	if(!inputChoice.selected(S.uniformlyRandom))
		placeElement(biasedLabel,biasedField);
	if(inputChoice.selected(S.isolatedCluster))
		placeElement(clusterLabel,clusterField);
	placeElement(showTransmit);
	placeElement(showEdges);
	placeElement(showLattice);
	if(showLattice.checked()) {
		placeElement(latticeBoxLabel,latticeBoxField);
		placeElement(latticeBoxSelectChoice);
		placeElement(latticeBoxDisplayChoice);
		};
	placeElement(computeBridges);
	if(computeBridges.checked()) {
		placeElement(bridgeButton);
		placeElement(phaseButton);
		};
	placeElement(showPowerPlot);
	placeElement(showCongestionPlot);
	placeElement(showExplanation);
	}

boolean concernsBiased(int scene) {
	boolean result = false;
	if(scene == guide.uniformRadiusBiased) result = true;
	else if(scene == guide.NNGBiased)             result = true;
	else if(scene == guide.NNGStrongBiased)       result = true;
	else if(scene == guide.lowDegreeStrongBiased) result = true;
	return result;
	}

boolean concernsBiased(int oldScene, int newScene) {
	boolean result = false;
	if(oldScene != newScene) {
		if     (concernsBiased(oldScene)) result = true;
		else if(concernsBiased(newScene)) result = true;
		};
	return result;
	}

public void actionPerformed(ControlElement e) {
	super.actionPerformed(e);
	if(e == guideSectionChoice) {
		guide.nextScene(guideSectionChoice.getSelectedIndex());
		if(concernsBiased(guide.oldScene,guide.scene)) layer.resetNodes();
		else if(guide.scene == guide.bridgePlot) layer.resetNodes();
		else layer.resetCircles();
		}
        else if(e == guideStepButton) {
		guide.nextScene(true);
		if(concernsBiased(guide.oldScene,guide.scene)) layer.resetNodes();
		else layer.resetCircles();
		}
        else if(e == guideBackButton) {
		guide.nextScene(false);
		if(concernsBiased(guide.oldScene,guide.scene)) layer.resetNodes();
		else layer.resetCircles();
		}
        else if(e == morePowerButton) {
		layer.changeRadius(true);
		}
        else if(e == lessPowerButton) {
		layer.changeRadius(false);
		}

        else if(e == bridgeButton) {
		layer.network.computeDistance();

		layer.network.appendBridge(layer.network.findBridge());
		}
        else if(e == phaseButton) {
		layer.network.phase ++;
		}
        else if(e == showTransmit) {
		layer.network.showTransmit = showTransmit.getState();
		layer.network.drawWideEdges = layer.network.showTransmit;
		}
	else if(e == showEdges) {
		}
	else if(e == latticeBoxField) {
		layer.network.newLattice();
		}
	else if(e == newInput) {

		layer.resetNodes();
		}
        else if(e == nField) {
		layer.resetNodes();
		}
        else if(e == biasedField) {
		layer.resetNodes();
		}
        else if(e == clusterField) {
		layer.resetNodes();
		};
	}

public void setDisabled() {
	guideStepButton.setDisabled(guide.scene >= guide.lastScene);
	guideBackButton.setDisabled(guide.scene <= 0);
	lessPowerButton.setDisabled(layer.network.largestRadius() <= P.nodeLargeRadius);
	morePowerButton.setDisabled(layer.network.connected());
	}


}
