package Delis;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.Math;
import java.util.Random;
import Library.*;
import Library.History.*;
import Library.Geometry.*;
import Library.Gadgets.*;

public class DelisLayer extends Layer {
	Delis             parent;
	DelisControlBar   control;
	DelisGuide        guide;
	DelisParameters   P;
	DelisStrings      S;
	DelisGuideStrings GS;


	WirelessNetwork   network;

	double            uniformPower;
	double            uniformCongestion;

	Plot              powerPlot;
	Plot              congestionPlot;
	PlotCurve         powerCurveUniform;
	PlotCurve         powerCurve;
	PlotCurve         congestionCurveUniform;
	PlotCurve         congestionCurve;
	int               nBridges;

	R2D               biasedRectangle;
	R2D               biasedRectangle4;

	DelisExplanation  explanation;
	R2D               explanationRectangle;

public void setup(UniString name, Rectangle rectangle) {
	super.setup(name, rectangle);
	network = new WirelessNetwork(this);
	explanation = new DelisExplanation(this);
	resetNodes();
	}

void setArea(C2D origin, double scale) {
	network.paintRectangle = new R2D(origin.x,origin.y,scale,scale);
	}

void resetNodes() {
	network.node = new WirelessNode[control.nField.getIntValue()];
	Point box = null;
	int clusterPoints = 0;
	biasedRectangle = null;
	if(control.inputChoice.selected(S.isolatedPoint))
		clusterPoints = 1;
	else if(control.inputChoice.selected(S.isolatedCluster))
		clusterPoints = control.clusterField.getIntValue();
	if(clusterPoints>0) {
		double biasedSize = control.biasedField.getIntValue();
		if(biasedSize < 0) biasedSize = 0;
		if(biasedSize > 50) biasedSize = 50;
		biasedSize *= 0.01;
		double biasedx = (1-biasedSize) * Math.random();
		double biasedy = (1-biasedSize) * Math.random();
		R2D r = new R2D(biasedx,biasedy,biasedSize,biasedSize);
		biasedRectangle  = r;
		biasedRectangle4 = new R2D(r.x+3*r.w/8,r.y+3*r.h/8,r.w/4,r.h/4);

		for(int i=0; i<clusterPoints; i++) 
			network.node[i] = createNode(distantPosition(),i);
		};
	for(int i=clusterPoints; i<network.node.length; i++) {
		C2D position = randomPosition();
		if(biasedRectangle != null) while(biasedRectangle.contains(position)) position = randomPosition();
		network.node[i] = createNode(position,i);
		};
	network.drawWideEdges = network.showTransmit;
	if(guide.scene==guide.bridgePlot) getUniformData();
	resetCircles();
	setPlot();
	nBridges = 0;
	network.lattice.considerNodes();
	}

void resetCircles() {
	for(int i=0; i<network.node.length; i++) {
		network.node[i].transmitRadius = getRadius();
		network.node[i].degree = 0;
		};
	network.drawWideEdges = network.showTransmit;
	network.edgesInvalid = true;
	network.firstBridge = null;
	}

void setPlot() {
	double yScale;
	yScale = 0.9; yScale /= uniformPower;
	powerPlot              = new Plot(new R2D(0.8,0.2,0.4,0.3),S.power);
	powerPlot.yScale = yScale;
	powerPlot.xScale = 0.05;
	powerCurveUniform      = new PlotCurve(powerPlot);
	powerPlot.add(powerCurveUniform);
	powerCurve             = new PlotCurve(powerPlot);
	powerPlot.add(powerCurve);

	yScale = 0.5; yScale /= uniformCongestion;
	congestionPlot         = new Plot(new R2D(0.8,0.6,0.4,0.3),S.congestion);
	congestionPlot.yScale = yScale;
	congestionPlot.xScale = 0.05;
	congestionCurveUniform = new PlotCurve(congestionPlot);
	congestionPlot.add(congestionCurveUniform);
	congestionCurve        = new PlotCurve(congestionPlot);
	congestionPlot.add(congestionCurve);

	powerCurveUniform.color = Color.red;
	powerCurveUniform.setValue(0.0,uniformPower);
	powerCurveUniform.setValue(1.0/powerPlot.xScale,uniformPower);

	powerCurve.color = Color.blue;

	congestionCurveUniform.color = Color.red;
	congestionCurveUniform.setValue(0.0,uniformCongestion);
	congestionCurveUniform.setValue(1.0/congestionPlot.xScale,uniformCongestion);

	congestionCurve.color = Color.blue;
	}

C2D distantPosition() {
	R2D r4 = biasedRectangle4;
	return new C2D(r4.x+r4.w*Math.random(),r4.y+r4.h*Math.random());
	}

C2D randomPosition() {
	return new C2D(Math.random(),Math.random());
	}

double getRadius() {
	double result = 0;
	UniString variant = control.modelChoice.getSelectedItem();
	if(control.modelChoice.selected(S.randomRadius))  result = 0.01*Math.random();
	if(control.modelChoice.selected(S.uniformRadius)) result = P.nodeLargeRadius;
	return result;
	}

WirelessNode createNode(C2D position, int ID) {
	WirelessNode n = new WirelessNode(network,ID);
	n.position = position;
	n.transmitRadius = getRadius();
	return n;
	}

boolean proportionalRadius() {
	if(control.modelChoice.getSelectedItem() == S.randomRadius)  return true;
	if(control.modelChoice.getSelectedItem() == S.uniformRadius) return true;
	return false;
	}

boolean nngRadius() {
	if(control.modelChoice.getSelectedItem() == S.nng)  return true;
	if(control.modelChoice.getSelectedItem() == S.lowDegree) return true;
	return false;
	}

void changeRadius(boolean increase) {
	network.changeRadius(increase);
	screen.history.setAnimation(true,History.END_LEVEL,History.LOW_LEVEL);
	}

boolean getUniformData(double r) {
	resetCircles();
	for(int i=0; i<network.node.length; i++) network.node[i].transmitRadius = r;
	network.initializeComponents();
	network.determineEdges();
	if(network.node[0].componentSize() < network.size()) return false;
	network.computeDistance();
	uniformPower = network.determinePower();
	uniformCongestion = network.determineCongestion();
	return true;
	}

void getUniformData() {
	double maxr = 1.0;
	double minr = 0.0;
	while(!getUniformData(maxr)) maxr *= 2.0;
	while(maxr-minr > 0.001) {
		double r = (maxr + minr) / 2;
		if(getUniformData(r)) maxr = r; else minr = r;
		};
	}

double round(double x) {
	int ix = (int) (100*x);
	double dx = ix; dx /= 100;
	return dx;
	}

boolean bridgeExists() {return network.firstBridge != null;}

boolean showPlots() {
	if(!bridgeExists()) return false;
	if(control.showPowerPlot     .checked()) return true;
	if(control.showCongestionPlot.checked()) return true;
	return false;
	}

void setArea() {
	if(showPlots()) setArea(new C2D(0.1,0.1),0.5);
	else            setArea(new C2D(0.05,0.1),0.8);
	}

public void paintOpening() {
	DelisTexts texts = new DelisTexts(this);
	texts.paint(pb,0);
	}

public void paint() {
	if(guide.scene == guide.opening) {paintOpening(); return;};
	R2D pbr = pb.getRectangle();
	double erw = 0.7;
	explanationRectangle = new R2D(pbr.x+pbr.w-erw-0.025,0.4,erw,0.5);
	setArea();
	network.paintBackground(pb);
	if(biasedRectangle != null) {
		int g;
		g = 250;
		pb.setColor(g,g,g);
		pb.fillRect(network.rectangle2paint(biasedRectangle));
		g = 240;
		pb.setColor(g,g,g);
		pb.fillRect(network.rectangle2paint(biasedRectangle4));
		};
	if(network.firstBridge != null) if(guide.scene == guide.bridgePlot) {
		powerPlot.paint(pb);
		congestionPlot.paint(pb);
		pb.setFont(0.02);
		pb.setColor(Color.blue);
		pb.drawString(""+round(uniformPower)+" | "+round(network.determinePower()),0.8,0.18);
		pb.drawString(""+round(uniformCongestion)+" | "+round(network.determineCongestion()),0.8,0.58);
		};
	network.paint(pb);
//	if(control.showExplanation.checked()) explanation.paint(pb);
	}


}

