package Delis;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.Math;
import Library.*;
import Library.Graph.*;
import Library.Geometry.*;

public class WirelessLatticeBox {
	WirelessLattice lattice;
	Point index;
	R2D rectangle;
	R2D centralRectangle;
	int nPoints;
	int nCenterPoints;
	
public WirelessLatticeBox(WirelessLattice lattice, Point index) {
	this.lattice = lattice;
	this.index = index;
	forgetPoints();
	setRectangle();
	}

public void forgetPoints() {
	nPoints = 0;
	nCenterPoints = 0;
	}

public void considerPoint(C2D position) {
	nPoints ++;
	if(centralRectangle.contains(position)) nCenterPoints++;
	}

public boolean centerFull() {
	return nCenterPoints > 0;
	}

public boolean centerCluster(int n) {
	return nCenterPoints >= n;
	}

public boolean boundaryEmpty() {
	return nCenterPoints == nPoints;
	}

public boolean emphasized() {
	return centerFull() && boundaryEmpty();
	}

public void setRectangle() {
	double boxSize  = lattice.boxSize;
	double cellSize = lattice.cellSize;
	double x = boxSize * index.x;
	double y = boxSize * index.y;
	rectangle = new R2D(x,y,boxSize,boxSize);
	double s = (boxSize - cellSize)/2;
	centralRectangle = new R2D(x+s,y+s,cellSize,cellSize);
	}

Color color() {
	if(emphasized()) return new Color(255,170,170);
	if(((index.x+index.y)%2) == 0) return new Color(240,240,190);
	return new Color(220,220,170);
	}

public void paint(Paintbrush pb) {
	R2D r = lattice.network.rectangle2paint(rectangle);
	R2D m = lattice.network.rectangle2paint(centralRectangle);
	double boxSize  = lattice.network.dimension2paint(lattice.boxSize);
	double cellSize = lattice.network.dimension2paint(lattice.cellSize);
	pb.setColor(color());
	pb.fillRect(r);
	pb.setColor(new Color(240,240,240));
	pb.fillRect(m);
	pb.setColor(Color.black);
	pb.drawRect(r);
	}

}
