package Delis;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.lang.Math;
import Library.*;
import Library.History.*;
import Library.Geometry.*;

public class WirelessNode {
	WirelessNetwork network;
	int    ID;
	C2D    position;
	double transmitRadius;
	double nodeLargeRadius;
	double nodeSmallRadius;
	Color  nodeColor;
	Color  transmitColor;
	Color  boundaryColor;
	WirelessNode[] neighbor;
	int    degree;
	int    maxDegree = 256;

	int    componentSize;
	WirelessNode componentSuperior;

public WirelessNode(WirelessNetwork network, int ID) {
	this.network = network;
	this.ID = ID;
	transmitRadius = 0;
	nodeLargeRadius = network.layer.P.nodeLargeRadius;
	nodeSmallRadius = network.layer.P.nodeSmallRadius;
	nodeColor = Color.black;
//	transmitColor = new Color(0,255,0);
	transmitColor = new Color(0,255,0,128);
	boundaryColor = Color.black;
	initializeComponent();
	neighbor = new WirelessNode[maxDegree];
	degree = 0;
	}

public void initializeComponent() {
	componentSize = 1;
	componentSuperior = this;
	}

public WirelessNode componentRepresentative() {
	WirelessNode result = this;
	while(result.componentSuperior != result) result = result.componentSuperior;
	WirelessNode c1, c2;
	c1 = this;
	while(c1 != result) {
		c2 = c1.componentSuperior;
		c1.componentSuperior = result;
		c1 = c2;
		};
	return result;
	}

public int componentSize() {
	return componentRepresentative().componentSize;
	}

double nodeRadius() {
	if(network.layer.control.showTransmit.checked())
		return network.dimension2paint(nodeLargeRadius);
	if(network.layer.control.showLattice.checked())
		return network.dimension2paint(nodeLargeRadius);
	return network.dimension2paint(nodeSmallRadius);
	}

public void paintNode(Paintbrush pb) {
//paintID(pb);
	if(nodeColor != null) {
		pb.setColor(nodeColor);
		C2D pos = network.position2paint(position);
		double r = network.dimension2paint(nodeRadius());
		pb.fillCircle(pos,r);
		};
	}

public void paintTransmit(Paintbrush pb) {
//p.p("node "+ID+" paintTransmit "+transmitRadius);
	if(transmitColor != null) {
		pb.setColor(transmitColor);
		C2D pos = network.position2paint(position);
		double r = network.dimension2paint(transmitRadius);
		pb.fillCircle(pos,r);
		};
	}

public void paintBoundary(Paintbrush pb) {
	if(boundaryColor != null) {
		pb.setColor(boundaryColor);
		C2D pos = network.position2paint(position);
		double r = network.dimension2paint(transmitRadius);
		pb.drawCircle(pos,r);
		};
	}

public void paintID(Paintbrush pb) {
	pb.setFont(0.05);
	pb.setColor(Color.black);
	pb.drawString(""+ID+"-"+componentSize(),network.position2paint(position));
//	pb.drawString(""+ID,paintPosition());
	}

static double distance(WirelessNode n1, WirelessNode n2) {
	return C2D.distance(n1.position,n2.position);
	}

boolean lowDegree() {
	DelisGuide guide = network.layer.guide;
	int variant = guide.scene;
	if(variant == guide.lowDegree) return true;
	if(variant == guide.lowDegreeStrongBiased) return true;
	if(variant == guide.bridge) return true;
	return false;
	}

void getNewNeighbor() {
	if(lowDegree()) if(componentSize() > network.size()/2) return;
	int bestNode = -1;
	double bestDistance = 0;
	for(int i=0; i<network.size(); i++) if(i != ID) {
		boolean isNeighbor = false;
		for(int j=0; (j<degree) && !isNeighbor; j++)
			if(neighbor[j] == network.node[i]) isNeighbor = true;
		double distance = distance(network.node[i],this);
		if(!isNeighbor) if((bestNode<0) || distance < bestDistance) {
			bestNode = i;
			bestDistance = distance;
			};
		};
	neighbor[degree++] = network.node[bestNode];
	}

void changeRadius(double scale) {
	DelisLayer layer = network.layer;
	Screen screen = layer.screen;
	History history = screen.history;
	double newRadius = scale * transmitRadius;
	history.appendex(new WirelessNodeRadiusChange(history,network.layer,this,newRadius));
	}

void changeNeighbors(boolean increase) {
	DelisLayer layer = network.layer;
//	UniString variant = layer.control.guideSectionChoice.getSelectedItem();
	WirelessNode nextNode = null;
	if(increase) {
		if(degree >= maxDegree - 1) p.p("error");
		getNewNeighbor();
		}
	else	{
		if(degree > 0) degree --;
		};
	}

}
